/********************************************************************
RTC_03_timery.c                                          
Program pokazuje jak uywa jednoczenie wielu licznikw (timers)
odliczajcych czas. Liczniki odliczaj czas mierzony w minutach lub 
sekundach. Podstaw tego programu jest podprogram obsugi przerwania 
ukadu RTC. Rezultatu niezalenych licznikw to:

- co 1 sekund odwieana bdzie data oraz czas na wywietlaczu LCD
- co minut diody LED bd miga przez 5 sekund
********************************************************************/
#include <LPC213x.H>        //zbir definiujcy rejestry procesora 
#include "uart.h"            //zbir definiujcy funkcje potrzebne do                 
                            //obsugi wywietlacza LCD
#include <stdio.h>          //zbir definiujcy standardowe funkcje 
                            //wejcia i wyjcia

#define MaxTimerS 3         //liczba timerw sekundowy

int TimerS[MaxTimerS];      //tablica timerw sekundowych


void RTC_isr(void)__irq;    //funkcja obsugi przerwa od RTC
void RTC_init(void);        //funkcja inicjalizujca RTC

void reset_timers(void);          //resetowanie wszystkich timerw
void set_timerS(int i, int val);  //ustawienie okrelonego TimerS
void RTC_ToStr(char *s);          //wywietlenie czasu na LCD

int main(void) {

    int n = 0x00010000;
    int blink = 0;                //zmienna kontrolujca miganie diod
	char s_time[60];

    UART_init;                     //inicjalizowanie wywietlacza LCD                 

    IODIR1 = 0x00FF0000;   //ustawienie portw P1.16-P1.23 jako wychodzce
    IOSET1 = 0x00FF0000;   //wygaszenie podczonych do tych portw diod
     
    set_timerS(0,1);       //timer sekundowy co 1 
	 set_timerS(1,10);    

      RTC_init();            //inicjalizowanie RTC
       printf ("START\n"); 
	   RTC_ToStr(s_time);    // zamiana Daty i Czasu na ASCII
       printf("\r %s",s_time);             // drukowanie stringu ASCII   

    while(1) {             //ptla programu
		// ShowDateTime();     //data oraz godzina
		// for(i=0;i<0xffffff;i++);
        if (TimerS[0]==0) {
            set_timerS(0,1);    //co sekund wywietli si dokadna          
	        RTC_ToStr(s_time);    // zamiana Daty i Czasu na ASCII
            printf("\r %s",s_time);             // drukowanie stringu ASCII               
        }

        if (TimerS[1]==0){ 
		     set_timerS(1,2); 
			 set_timerS(2,1);    
             blink = 1;         //wyczenie migania diod
        }    

	     if (TimerS[2]==0){ 
	  
             blink = 0;         //wyczenie migania diod
        }   
  
        if(blink) IOSET1 = 0x00FF0000;  
		else 	IOCLR1 = 0x00FF0000;  
     
       
    }
}       

//inicjalizowanie ukadu RTC
void RTC_init(void) {
    int g,m,s;
    CCR   = 0x02;           //resetowanie zegara 
	scanf("%02d %02d %02d",&g,&m,&s);
    SEC   = s & 0x3f;            //ustawienie sekund
    MIN   = m & 0x3f;             //ustawienie minut
    HOUR  = g & 0x1f;             //ustawienie godziny
    DOM   = 1;              //ustawienie dnia miesica
    DOW   = 4;              //ustawienie dnia tygodnia
    DOY   = 121;            //ustawienie dnia roku
    MONTH = 5;              //ustawienie miesica
    YEAR  = 2006;           //ustawienie roku

    CIIR  = 0x01;           //przerwanie przy inkrementacji licznika SEC
  
    VICVectAddr13 = (unsigned)RTC_isr; //adres procedury obsugi przerwa
    VICVectCntl13 = 0x0000002D;        //ustalenie rda przerwa (D - RTC)
                                       //o priorytecie 13
    VICIntEnable  = 0x00002000;        //zezwolenie na przerwania od RTC
    ILR           = 0x00000003;        //zerowanie rejestru lokacji         
                                       //przerwa
    CCR=0x11;              //start RTC
    PCONP=0x1815BE;        //odcicie zegara VPB dla RTC 
                           //celem oszczdnoci energii 
}

// zerowanie licznikw odliczajcych czasy
void reset_timers(void) { 
    int i;
    for (i=0;i<MaxTimerS;i++)   
        TimerS[i]= 0;  
}  


// ustawienie czasu odliczania sekund ------------------------ 
void set_timerS(int i, int val) { 
    if(i < MaxTimerS) TimerS[i] = val;
}


//obsuga przerwania ukadu RTC
void RTC_isr(void)__irq
{
    int i;
    if(ILR & 0x00000001) {      //czy nastpio przerwanie od CIIR
	    //modyfikacja licznikw odliczajcych sekundy
        for(i=0;i < MaxTimerS;i++) 
            if (TimerS[i] > 0) TimerS[i]--;
      
    }
    i           = ILR;  //odczytanie zawartoci ILR
    ILR         = i;    //zerowanie rejestru lokacji przerwa
    VICVectAddr = 0;    //koniec przerwania
}

//zamia struktury tim na ASCII string dd/mm/yy hh:mm:ss ----------------
void RTC_ToStr(char *s) {
	//
    const char *SWeekday[7] = {
        "Nie", "Pon", "Wto", "Sro", "Czw", "Pia", "Sob"};
        sprintf(s,"%s %02d/%02d/%04d   %02d:%02d:%02d     (%03d)",
           SWeekday[DOW & 0x7], DOM & 0x1f, MONTH & 0xf,
             YEAR & 0xfff,HOUR & 0x1f, MIN & 0x3f, SEC & 0x3f,DOY & 0xfff);
}
