/**********************************************************************
led_blink_THUMB.s                                   (mruganie diod LED)

Przykadowy program w jzyku asembler procesora LPC2138.
Program powoduje mruganie 7-miu diod LED. Przycinicie przycisku
przerwania wywouje przerwanie FIQ co objawia si zapaleniem diody LED
numer 8.Program pokazuje jak wywoa procedur THUMB.
**********************************************************************/
$include (LPC214x.h)      // zbir definicji rejestrw taki sam jak w C
#define Nbit  16                     //diody LED przesunite o 16-bitw

AREA    STARTUPCODE, CODE, AT 0 //obszar kodu programu zaczyna si od 0

SFRreg    MACRO   reg, val           // makrodefinicja wpisujca do ...
          MOV R8, val                   // ... rejestru reg warto val
          LDR R9,=reg                  
          STR R8,[R9,#0x0] 
          ENDM
         
start     PROC    CODE32  // procedura w kodzie ARM -------------------

          B         Reset_Handler         // wektory zdarze (przerwa)
          B $      ;Undef_Handler 
          B $      ;SWI_Handler 
          B $      ;PAbt_Handler 
          B $      ;DAbt_Handler 
          NOP      ;Reserved Vector 
          B $      ;IRQ_Handler 
          B         FIQ_Handler               // obsuga przerwania FIQ

Reset_Handler:                               // obsuga zdarzenia RESET
          SFRreg VICIntSelect,#0x00008000 //przerwanie od wejcia P0.14
          SFRreg VICIntEnable,#0x00008000    // odblokowanie przerwania
          SFRreg PINSEL0,     #0x20000000     // bit P0.14 obsuguje...
                                              //...          przerwanie
          SFRreg IODIR1,      #(0xff<<Nbit) //8-bitw diod LED jako OUT
          SFRreg IOSET1,      #(0xff<<Nbit)       // zgaszenie diod LED

          MRS   R0, CPSR                         // odczyt rejestru PSW
          EOR   R0,R0, # 1<<6    // negacja bitu obsugi przerwania FIQ
          MSR   CPSR_c, R0               // odblokowanie przerwania FIQ
          LDR   SP, =0x40000100    // przygotowanie stosu w pamici RAM
          MOV   R0,#1      // rejestr R0 bdzie sterowa mruganiem diod
         
loop:    
            BL delay              // podprogram opnienia czasowego (1)
            MOV R0,R0, ROR #1 //licznik kodu piercieniowego: 1 z 32 (1)
           ;ADD R0,R0,#10               // 32-bitowy licznik binarny (2)
            AND R1,R0,    #(0x7f<<Nbit)              // 7-diod LED mruga
            SFRreg IOSET1,#(0x7f<<Nbit)            // zgaszenie diod LED
            SFRreg IOCLR1,R1                       // zapalenie diod LED

            B   loop                                  // ptla bez koca
            ENDP

FIQ_Handler PROC  CODE32   // obsuga przerwania FIQ ------------- -----
            SFRreg IOCLR1,#(1<<(Nbit+7) ) // zapalenie diody LED numer 8
            SFRreg IOSET1,#(1<<(Nbit+7) ) // zgaszenie diody LED numer 8
            SFRreg EXTINT,#0x00000002      // odblokowanie kontrolera...
                                           //...                przerwa
            SUBS PC,LR,#4          // wyjcie (RET) z obsugi przerwania
            ENDP

delay       PROC    CODE32  //  procedura ARM wywoujca procedur THUMB 
            LDR R12, [R15]
            BX  R12
            DD delayT
            ENDP
         

delayT      PROC    CODE16 // procedra THUMB ---------------------------
            PUSH {LR}                       // zapis na stos rejestru LR
            LDR  R7,=0x5000                            //czas opnienia
delay_00:   SUB  R7,#1                             // odliczanie "w d"
            BNE   delay_00                           // ptla opnienia
            POP  {R3}                                 // odczyt ze stosy
            BX   R3                       // wyjcie (RET) z podprogramu
            ENDP

            END       