/**********************************************************************
counter.c                           (licznik binarny na diodach LED ...
                           ... lub diodach 8-segmentowego wywietlacza)

Program pokazuje dziaanie licznika binarnego dziaajcego na 8-miu 
bitach portu P1.  W takt naciskania przycisku Button (P0.14)zwikszana
jest zawarto licznika. Diody LED doczone s do 
bitw P1.23..P1.16 portu P1. 
**********************************************************************/
#include <LPC214x.H>       //zbir definiujcy rejestry mikrokontrolera
#include <INTTYPES.H>         //zbir definiujcy cakowite typy danych

#define NBit_LED    18               // przesunicie dla 8-miu diod LED
#define NBit_BUT    14                        //pooenie przycisku BUT
                                      
#define M           0xFF                     //maska dla 8-miu diod LED

#define LEDs_init IO1DIR = IO1DIR | (M << NBit_LED )     // port OUTPUT
#define LEDs_clr  IO1CLR = M << NBit_LED           // wyzeruj diody LED
#define LEDs_set  IO1SET = M << NBit_LED             // wcz diody LED
          
// podprogram wywietla stan zmiennej x na diodach LEDs ---------------
inline void LEDs_put( uint8_t x) {
  IO1CLR = ( ~x & M ) << NBit_LED ;       // zerowanie wskazanych bitw
  IO1SET = (  x & M ) << NBit_LED ;      // ustawianie wskazanych bitw
}

// podprogram sprawdzanie stanu bitu INPUT ----------------------------
inline  uint8_t  BUT_get (void ) {	                          
  return( (IO0PIN & ( 1<< NBit_BUT  ) ) >> NBit_BUT); // zwraca 0 lub 1
}                

// podprogram opnienia czasowego ------------------------------------
void delay ( uint32_t x){                       // x to czas opnienia
  uint32_t i;   //deklaracja zmiennej, ktra posuy jako licznik ptli
  for(i = 0; i <= x; i++){};                       //opnienie czasowe
}

//---------------------------------------------------------------------
int main (void)  {
 uint8_t counter;

 LEDs_init;
 while(1){  
     while(   BUT_get())delay(0xFF);         // czekanie na nacinicie
     while( ! BUT_get())delay(0xFF);          // czekanie na zwolnienie
     LEDs_put(~counter++);  //zwikszanie, wywietlanie i negacja bitw
 } 
}