/**********************************************************************
J_cnt.c                 (licznik piercieniowy Johnsona na diodach LED)

Program pokazuje dziaanie licznika piercieniowego Johnsona na 8-miu
bitach portu P1. W takt naciskania  przycisku Button (P0.14) zapala 
si kolejna jedna dioda pord pozostaych zgaszonych. Diody LED 
doczone s do bitw P1.23..P1.16 portu P1. 
**********************************************************************/
#include <LPC214x.H>       //zbir definiujcy rejestry mikrokontrolera
#include <INTTYPES.H>         //zbir definiujcy cakowite typy danych

#define NBit_LED    16               // przesunicie dla 8-miu diod LED
#define NBit_BUT    14                        //pooenie przycisku BUT
                                      
#define M           0xFF                     //maska dla 8-miu diod LED

#define LEDs_init IO1DIR = IO1DIR | (M << NBit_LED )     // port OUTPUT
#define LEDs_clr  IO1CLR = M << NBit_LED           // wyzeruj diody LED
#define LEDs_set IO1SET = M << NBit_LED              // wcz diody LED
          
// podprogram wywietla stan zmiennej x na diodach LEDs ---------------
inline void LEDs_put( uint8_t x) {
  IO1CLR = ( ~x & M ) << NBit_LED ;       // zerowanie wskazanych bitw
  IO1SET = (  x & M ) << NBit_LED ;      // ustawianie wskazanych bitw
}

// podprogram sprawdzanie stanu bitu iNPUT ---------------------------------
inline  uint8_t  BUT_get (void ) {                          
  return( (IO0PIN & ( 1<< NBit_BUT  ) ) >> NBit_BUT); // zwraca 0 lub 1
}                

// podprogram opnienia czasowego ------------------------------------
void delay ( uint32_t x){                       // x to czas opnienia
  uint32_t i;   //deklaracja zmiennej, ktra posuy jako licznik ptli
  for(i = 0; i <= x; i++){};                       //opnienie czasowe
}

// licznik 8-bitowy w kodzie Johnsona (tryb ARM) ----------------------
uint8_t    johnson_A( uint8_t i) __arm {
  __asm {
      MOV  R1,#0           // zerowanie rejestru R1
      MOVS  R0,R0, LSL #25 // rejestr R0 zawiera parametr i
                           // ustaw flag C przy SHIFT
      ADC  R1,R1,R1        // rejestr zawiera najstarszy bit
      mov R4,#1            // maska 0x01
      EOR R1,R4,R1         // negowanie bitu
      MOV R0,R0, LSR #24   // przesuwanie w prawo
      ADD R0,R1,R0         // dodaj zanegowany bit
  }
}

// licznik 8-bitowy wkodzie Johnsona (tryb THUMB) ---------------------
uint8_t  johnson_T(uint8_t i) __thumb {
   __asm {
      mov R1,#0        // zerowanie rejestru R1
      LSL R0,R0, #25   // rejestr R0 zawiera parametr i
                       // ustaw flag C przy SHIFT
      ADC R1,R1        // rejestr zawiera najstarszy bit
      mov R4,#1        // maska 0x01
      EOR R1,R4        // negowanie bitu
      LSR R0,R0, #24   // przesuwanie w prawo
      ADD R0,R1        // dodaj zanegowany bit
   }
}

//---------------------------------------------------------------------
int main (void)  {
 uint8_t cnt=1;

 LEDs_init;
 while(1){  
     while(   BUT_get())delay(0);  // czekanie na nacinicie przycisku
     while( ! BUT_get())delay(0);   // czekanie na zwolnienie przycisku
 //cnt= ( cnt & 0x80 ) ? (cnt<<1):(cnt<<1) | 1; // licznik Johnsona (1)
 //cnt=johnson_A(cnt);      // licznik Johnsona: wywoanie proc ARM (2)
       cnt=johnson_T(cnt);// licznik Johnsona: wywoanie proc THUMB (3)
       LEDs_put(~cnt);                  // negacja bitw i wywietlanie
 } 
}