/**********************************************************************
T_measure.c                                     (pomiar okresu sygnau)

Program demonstruje wykorzystanie licznika Timer0 do pomiaru okresu 
podawanych na wejcie CAP0.0 (P0.2) impulsw. Warto pomiaru, 
wyraona w mikrosekundach, wywietlana jest na podczonych do 
portu P1.31...P1.16 szesnastu diodach LED. Narastajce zbocze
tego sygnau na wejcie CAP0.0 (P0.2)wywouje przerwanie.
W podprogramie obsugi przerwania wywietla si stan ukadu
Timer0, zeruje si stan Timer0 i rozpoczyna proces zliczanie
impulsw wejciowych przez Timer0.
Sygna pomiarowy o zadanej czstotliwoci mona uzyska zwierajc
piny P0.2 i P0.1 oraz nadajc za pomoc programu Hyperterminal.exe
cig liter UUUU... Przy szybkoci transmisji 9600bps wynik
wywietlany na diodach LED to 204usek. (0xd0).
**********************************************************************/
#include <LPC214x.H>            // zbir definiujcy rejestry procesora

void T0isr(void) __irq;                  // prototyp obsugi przerwania

void main(void) {

  IODIR1 =  0xFFFF0000;                      // P1.16 - 31 jako wyjcia

  PINSEL0 = PINSEL0 | (1 << 5);                     // P0.2 -->  CAP0.0

  T0TCR = 0x00000002;                // zerowanie licznika i preskalera
                                      //...do momentu zdjcia tego bitu
  T0PR  = 14;                   // 1us (ustawienie odliczanej wartoci)
  T0CCR = 0x00000005;       // zachowanie wartoci TC w CR0 i wywoanie
                           // przerwania przy zboczu rosncym na CAP0.0
  T0TCR = 0x00000001;                               // wczenie Timer0

  while (1) {
    PCON = 0x00000001;                         // przejcie w tryb Idle
  }
}

//--------------------------------- procedura obsugi przerwania Timer0
void T0isr (void) __irq {
  unsigned int period;

  T0TCR = 0x00000002;                  // zerowanie Timer0 i preskalera
  T0TCR = 0x00000001;                               // wczenie Timer0

  period = T0CR0 + 2;                                        // korekta
  
  IOCLR1 = ~period << 16;                         // zgaszenie diod LED
  IOSET1 = period << 16;                          // zapalenie diod LED

  T0IR |= 0x00000001;                     // zerowanie flagi przerwania
  VICVectAddr = 0x00000000;                                   // powrt
}