//------------------------------------------------------------------------
//zbir: LCD_comm.c 
//------------------------------------------------------------------------

// LCD_set_cursor - ustaw kursor: wiersz, kolumna-------------------------
void LCD_set_cursor (char rownr, char colnr) {
  int cmd;
  rownr= rownr%NB_ROW;      // umieszczenie numeru wiersza obszarze limitu
  colnr= colnr%NB_COL;      // umieszczenie numeru kolumny obszarze limitu
  switch (rownr) {
    case 0:
           cmd = ROW0 | 0x80;
           break;
    case 1:
          cmd = ROW1  | 0x80;
          break;
    case 2:
          cmd = ROW2  | 0x80;
          break;
    case 3:
          cmd = ROW3  | 0x80;
          break;
    default:
           cmd = ROW0 | 0x80;
           break;
  }
  LCD_ctrlWR(cmd + colnr); 
}

// get cursor - odczyt pozycji kursora-----------------------------------
void get_cursor (char *rownr, char *colnr) {
  char x;
  x=LCD_ctrlRD() ;              // odczyt biecego adresu pamici DD LCD

  *rownr =NB_ROW-1;
  *colnr=0;
#if NB_ROW>=4
  if(x>= ROW3 ) { *rownr =3; *colnr=x-ROW3; return;}
#endif

#if NB_ROW>=3
  if(x>= ROW2 ) { *rownr =2; *colnr=x-ROW2; return;}
#endif

#if NB_ROW>=2
  if(x>= ROW1 ) { *rownr =1; *colnr=x-ROW1; return;}
#else
  if(x>= ROW0 ) { *rownr =0; *colnr=x-ROW0; return;}
#endif

}

// lcd_init - inicjacja wywietlacza-------------------------------------
void lcd_init (void) {
  char i;
  unsigned j;
                 // sekwencja inicjacji: 8/4-bitowy interfejs,  5*7 znak,
                 //     +1 inkrementacja, wcz wywietlacz, wcz kursor

  const char init_tab[] = 
       {0x30,0x30,INTERFACE,0x01, 0x0F};
  const char  fonttab[] = {                      //tablica polskich znakw
                	    0, 0,14,01,15,17,15, 2,   // 
			    0, 0,14,17,31,16,14, 2, 	// 
			    2, 4,14,16,16,17,14, 0, 	// 
			    2, 4,14,16,14, 1,30, 0,  	// 
			    4, 0,31, 2, 4, 8,31, 0 , 	// ,
			   12, 4, 6, 4,12, 4,14, 0, 	// 
			    2, 4,22,25,17,17,17, 0, 	// 
			    2, 4,14,17,17,17,14, 0, 	// 
  };
  for (j = 0; j <0xfffff; j++) ; 
  LCD_all_init();

  for (i = 0; i < sizeof init_tab; i++) {
     LCD_ctrlWR(init_tab[i]);                 // zapis danych inicjujcych
  }

  LCD_ctrlWR(CG_RAM);                                    // wybierz CG RAM
  for (i = 0; i < sizeof fonttab; i++) {
     LCD_dataWR(fonttab[i]);                      // zaaduj polskie znaki
  }

LCD_ctrlWR(DD_RAM);    // wybierz DD RAM
        LCD_set_cursor(NB_ROW-1, 0);

}

// lcd_clear - kasowanie ekranu ------------------------------------------
void LCD_clear (void) {
 LCD_ctrlWR(CLR_LCD);
}

// lcd_cursorOFF - wyczenie kursora ------------------------------------
void LCD_cursorOFF (void) {
 LCD_ctrlWR(CUR_OFF);
}

// lcd_pl - interpretacja znakw do wywietlenia -------------------------
void lcd_pl (char c) {
char row,col,i,j,x;
switch (c) {

   case '\n':                         // nowa linia-----------------------
          for (j=0;j<NB_ROW-1;j++)              // rolowanie w gr tekstu
          for (i=0;i<NB_COL;i++) { 
          LCD_set_cursor(j+1,i);                    //przepisywanie tekstu
             x=LCD_dataRD();                     
             LCD_set_cursor(j,i);    
             LCD_dataWR(x);                    
           }
          LCD_set_cursor(NB_ROW-1, 0);
          for(i=0;i<NB_COL;i++) LCD_dataWR(' ');  //kasowanie linii tekstu
          LCD_set_cursor(NB_ROW-1, 0);
          break;
   case '\r':                         // powrt do pocztku linii---------
          get_cursor(&row, &col); 
          LCD_set_cursor(row,0);
          break;
   case 0x08:             // Back Space: cofanie o jeden znak ------------
        get_cursor(&row, &col);      
        if(col>0)  LCD_set_cursor(row, --col);                                                       
        break;

   case 0xb9:                         //polskie znaki -------------------
        LCD_dataWR(0); //
        break;
   case 0xea:
        LCD_dataWR(1); //
        break;
   case 0xe6:
        LCD_dataWR(2); //
        break;
   case 0x9c:
        LCD_dataWR(3); //
        break;
   case 0xbf:
        LCD_dataWR(4); //,
        break;
   case 0x9f:
        LCD_dataWR(4); //,
        break;
   case 0xb3:
        LCD_dataWR(5); //
        break;
   case 0xf1:
        LCD_dataWR(6); //
        break;
   case 0xf3:
        LCD_dataWR(7); //
        break;
   default:
        if (c>0x1f && c<0x80) LCD_dataWR(c);       // wywietlenie znaku       
        break;
 }
}

//funkcja wywietlania zgodna z opisem w stdio.h------------------------
int putchar(int c) {      
   lcd_pl (c);             
   return (c);
}
