/*********************************************************************
RTC_test.c                    (obsuga ukadu RTC  zegar i kalendarz)

Program pozwala ustawi oraz odczyta rejestry ukadu 
zegara-kalendarza RTC. Odczyt rejestrw sterowany jest przerwaniem od
zmiany sekund w RTC. Co 1 sekund drukowany jest aktualny czas, data,
dzie tygodnia i dzie roku. Co jedn sekund zapala si dioda LED. 
Wyniki wyprowadzane s na zcze transmisji szeregowej 
(38400bps 8-N-1). Nacinicie klawisza TAB umoliwia zmian daty i
czasu, Dziaanie ukadu RTC podtrzymywan jest dodatkow bateri.
Ukad RTC posiada zewntrzny oscylator kwarcowy 32kHz.
*********************************************************************/
#include <LPC213x.H>        
#include "uart.h"                            
#include <stdio.h>          
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

typedef struct {
  WORD   TM_sec;                                   // sekundy  [0, 59]
  WORD   TM_min;                                   // minuty   [0, 59]
  WORD   TM_hour;                                  // godziny  [0, 23]
  WORD   TM_dom;                               // dni miesica [1, 31]
  WORD   TM_month;                     // miesice od stycznia [0, 11]
  WORD   TM_year;                                 // lata od roku 1900
  WORD   TM_dow;                 // dzie tygodnia od niedzieli [0, 6]
  WORD   TM_doy;               // dni roku od 1-go stycznia - [0, 365]
} TM;

#define Nbit 23                                // numer bitu diody LED

void RTC_isr(void)__irq;                    // obsuga przerwa od RTC
void RTC_init(void);                                // inicjowanie RTC

void TM_to_RTC(TM *time);   // prototyp: data iczas TM -> RTC rejestry
void RTC_to_TM(TM *time);  // prototyp: RTC rejestry -> data iczas TM 
void TM_to_string(TM *tim, char *s);     // wywietlenie: TM -> string
void RTC_to_string(char *s);   // wywietlenie: RTC rejestry -> string
void set_RTC(TM *time);

BYTE flag, blink;                  //  flaga: czekanie przez 1 sekund
const char *SWeekday[7] = {                            // dni tygodnia
        "Nie", "Pon", "Wto", "Sro", "Czw", "Pia", "Sob"};

//--------------------------------------------------------------------
void main(void) {
    TM TIME;
    char string[60];

    UART_init;                                      //inicjowanie UART

    IODIR1 = 1<<Nbit;                         // dioda LED jako output
     
    RTC_init();                                     // inicjowanie RTC
    printf ("\n RTC DEMO ----------------------------------------\n");
    while(1) {                                 //ptla gowna programu

      if ( flag  ) {              // wywietlanie dokadnie co sekund
         RTC_to_string(string);      // zamiana rejestrw RTC na ASCII
         printf("\n %s",string);                         // drukowanie

         RTC_to_TM(&TIME);            
         TM_to_string(&TIME,string); //zmiana daty i czasu TM na ASCII
         printf("\n %s\n",string);                       // drukowanie
         flag=0;                                    // flaga 1 sekunry
      }

      if((U0LSR & 0x01))
              if( _getkey()== 0x09 ) set_RTC(&TIME);  // czekaj na TAB
    }
}

// inicjowanie ukadu RTC --------------------------------------------
void RTC_init(void) {
  CIIR  = 0x01;                // przerwanie przy zmianie licznika SEC
  VICVectAddr13 = (unsigned)RTC_isr;       // adres obsugi przerwania
  VICVectCntl13 = 0x0000002D;   // ustalenie rda przerwa (D - RTC)
                                                //... o priorytecie 13
  VICIntEnable  = 0x00002000;       // zezwolenie na przerwania od RTC
  ILR           = 0x00000001;       // przerwania od zmiany w rej. RTC
  PCONP         = 0x8;                           // dziaa tylko UART0
                                       //...celem oszczdnoci energii
}

// przepisywanie daty i czasu TM do rejestrw RTC --------------------
void TM_to_RTC(TM *x) {
    CCR   = 0x02;                              // reset i stop dla RTC
    SEC   = x->TM_sec   & 0x3f;                    //ustawienie sekund
    MIN   = x->TM_min   & 0x3f;                     //ustawienie minut
    HOUR  = x->TM_hour  & 0x1f;                   //ustawienie godziny
    DOM   = x->TM_dom   & 0x1f;             //ustawienie dnia miesica
    DOW   = x->TM_dow   & 0x7;              //ustawienie dnia tygodnia
    DOY   = x->TM_doy   & 0xfff;                //ustawienie dnia roku
    MONTH = x->TM_month & 0xf;                   //ustawienie miesica
    YEAR  = x->TM_year  & 0xfff;                     //ustawienie roku
    CCR=0x11;                                              //start RTC
}

// przepisywanie rejestrw RTC do daty iczasu TM ---------------------
void RTC_to_TM(TM *x) {
    x->TM_sec   =  SEC   & 0x3f;                      // odczyt sekund
    x->TM_min   =  MIN   & 0x3f;                       // odczyt minut
    x->TM_hour  =  HOUR  & 0x1f;                     // odczyt godziny
    x->TM_dom   =  DOM   & 0x1f;               // odczyt dnia miesica
    x->TM_dow   =  DOW   & 0x7;                // odczyt dnia tygodnia
    x->TM_doy   =  DOY   & 0xfff;                  // odczyt dnia roku
    x->TM_month =  MONTH & 0xf;                     // odczyt miesica
    x->TM_year  =  YEAR  & 0xfff;                       // odczyt roku
}

//obsuga przerwania ukadu RTC od zmiany sekund ---------------------
void RTC_isr(void)__irq  {
    if(ILR & 0x00000001) {          //czy nastpio przerwanie od CIIR

      flag=1;                                // upyna jedna sekunda
      if ((blink++)&1)  IOCLR1 = 1<<Nbit;       // zapalenie diody LED
      else             IOSET1 = 1<<Nbit;        // zgaszenie diody LED
    }
    ILR         = ILR;         //zerowanie przerwania od zmiany sekund
    VICVectAddr = 0;                              // koniec przerwania
}

//drukowanie rejestrw RTC jako: dd/mm/yy hh:mm:ss -------------------
void RTC_to_string(char *s) {
   sprintf(s,"%s %02d/%02d/%04d   %02d:%02d:%02d     (%03d)  RTC",
      SWeekday[DOW & 0x7], DOM & 0x1f, MONTH & 0xf,
        YEAR & 0xfff,HOUR & 0x1f, MIN & 0x3f, SEC & 0x3f,DOY & 0xfff);
}

//zamia struktury TM na string dd/mm/yy hh:mm:ss ---------------------
void  TM_to_string(TM *tim, char *s) {
 sprintf(s,"%s %02d/%02d/%04d   %02d:%02d:%02d     (%03d)  TM",
   SWeekday[tim-> TM_dow], tim->TM_dom,tim->TM_month,
    (tim->TM_year),tim->TM_hour,tim->TM_min,tim->TM_sec, tim->TM_doy);
}

// ustawianie rejestrw RTC ------------------------------------------
void set_RTC(TM *time) {
   WORD hh,mm,ss,dd,MM,yyyy,day_w,day_y;
   char string[60];
   printf("\n\n hh mm ss   <- ");
   scanf("%2u %2u %2u", &hh, &mm, &ss );  
   printf("\n dd MM yyyy <- ");
   scanf("%2u %2u %4u", &dd, &MM, &yyyy ); 
   printf("\n 0- Nie, 1-Pon 2-Wto 3-Sro 4-Czw 5-Pia 6-Sob  <- ");
   scanf("%1u", &day_w ); 
   printf("\n dzien roku (1...356) <- ");
   scanf("%3u", &day_y);               

   time->TM_sec   = ss;                            //ustawienie sekund
   time->TM_min   = mm;                             //ustawienie minut
   time->TM_hour  = hh;                           //ustawienie godziny
   time->TM_dom   = dd;                     //ustawienie dnia miesica
   time->TM_doy   = day_y;                      //ustawienie dnia roku
   time->TM_dow   = day_w;                      //ustawienie dnia roku
   time->TM_month = MM;                          //ustawienie miesica
   time->TM_year  = yyyy;                        //ustawienie miesica

   TM_to_RTC (time);                 // kopiowanie TM do rejestrw RTC

   RTC_to_string(string);            // zamiana daty i czasu na string
   printf("\r %s",string);                       // drukowanie stringu
}    