/***************************************************************************
SPI_eeprom.c                       (zapis i odczyt do pamici EEPROM  25040)

Program zapisuje i odczytuje dane do pamici EEPROM 25010,25020 i 25040 z
interfejsem SPI. Procesor steruje ukadowym interfejsem SPI wystpujcym w
procesorach Philips LPC lub obsuguje interfejs SPI metod programow
(softrware'owo). Aby wybra sposb obsugi interfejsu SPI wystarczy wymieni
modu programu
       spi0_hard.c - obsuga ukadowa interfejsu SPI0
       spi0_soft.c - obsuga programowa (software'owa) SPI0.
Na projekt skadaj sie:
   SPI_eeprom.c               - modu gwny obsugi pamici EEPROM SPI
   spi0_hard.c / spi0_soft.c  - biblioteka SPI
   at25040.c                  - biblioteka podprogramw obsugi 
                                pamici EEPROM.
Zapis i odczyt do pamici odbywa si dwoma metodami. Pierwsza metoda to 
zapis i odczyt bajt po bajcie osobno. Druga metoda to zapis i odczyt 
blokami (stronami). Moliwo zapis i odczytu blokami zaley od typu 
pamici. Firma Xicor zezwala na zapis blokami typu WORD po dwa bajty.
Firma Atmel zezwala na zapis stronami po 8-bajtw.
Wyniki dziaania programu wyprowadzane s na zcze SIO: 38400bps 8-N-1.
***************************************************************************/
#include <LPC214x.H>                               // rejestry SFR procesora
#include "inttypes.h"                  // tu zdefiniowano BYTE, WORD i DWORD
#include "UART.h"                      // przygotowanie UART: 38400bps 8-N-1
#include "AT25040.h"                 // obsuga pamici 25010, 25020 i 25040
#include "spi.h"        // obsuga SPI interfejsu w procesorach '213x, '214x
#include <stdio.h>     

#define BUFFER_SIZE     255      

// ------------------------------------------------------------------------
void main(void) {

        BYTE adr;
        BYTE buf[BUFFER_SIZE];                 // bufor danych wysyanych i
                                                    // ...odbieranych z SPI
        UART_init;                      // przygotowanie SIO: 9600bps 8-N-1
        IODIR0 |= (1 << AT25040_CS_NBit )  ;  
 
        printf("\n\nBiblioteka SPI (obsuga - hardware'owa)\n");

        SPI_Init();  
  
        AT25040_clear_write_enable();       // odblokowanie zapisu do 25040
        
        printf("\nStatus: %02x",AT25040_read_status()); 
        AT25040_set_write_enable();         // zezwolenie na zapis do 25040
        printf("\nStatus: %02x",AT25040_read_status());

        printf("\nodczyt [0x55]: %02x", AT25040_RD_byte(0x55));
        printf("\nzapis  [0x55} = 0x55");

        AT25040_WR_byte(0x55,0x55);
        printf("\nodczyt [0x55]: %02x", AT25040_RD_byte(0x55));

        printf("\nzapis  [0x55] = 0xAA");
        AT25040_WR_byte(0x55,0xAA);
        printf("\nodczyt [0x55]: %02x", AT25040_RD_byte(0x55));

        printf("\n ZAPIS 0..255");
        
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) 
                                     AT25040_WR_byte(adr,adr);
        printf("\n ODCZYT 0..255\n");  
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) {
              printf("%02x ", AT25040_RD_byte(adr));
              if ( (adr %16) ==15)  printf("\n"); 
        }
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) buf[adr]=0;  
  
        printf("\n Cigy odczyt 0..255 \n");      
        AT25040_RD_data( 0, buf,  BUFFER_SIZE);
        for ( adr=0;adr < BUFFER_SIZE;adr++ ) {
             printf("%02x ",buf[adr]);
             if ( (adr %16) ==15)  printf("\n"); 
         }

        for ( adr=0; adr < BUFFER_SIZE; adr++ ) 
                                            buf[adr]=~adr;    // nowy bufor

        printf("\n\n Ciagly zapis 0..1 \n");        
        AT25040_WR_data( 0, buf, 2);          // zapis sowa word (2 bajty)
        printf("\n\n Ciagly odczyt 0..1 \n");       
        AT25040_RD_data( 0, buf, 2);              // odczyt sowa (2 bajty)
        for ( adr=0; adr < 2; adr++ ) { 
            printf("%02x ",buf[adr]);
            if ( (adr %16) ==15)  printf("\n"); 
        }

    while(1);
}