/*********************************************************************
1_wire.c                       (biblioteka procedur termometru 1-wire)

Biblioteka zwiera 3 procedury obsugi termometru DS1820 z interfejsem
1-wire Te procedury to:
- wysanie impulsu startu i czekanie na potwierdzenie:WIRE1_present
- zapis bajtu danych:                                 WIRE1_write_byte
- odczyt bajtu danych:                                WIRE1_read_byte

Wszystkie procedury zale od procedury opnienia:      delay_us.
Procedura ta jest krytyczna dla dziaania biblioteki obsugi 1-wire.
*********************************************************************/
#include <LPC214x.H>                    // LPC214x definicje rejestrw
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "1_wire.H"                              // prototypy procedur

// odczyt bajtu danych z 1-wire --------------------------------------
char WIRE1_read_byte() { 
   char i,data;

   for(i=1;i<=8;++i) {                               // odbir 8-bitw
     WIRE1_out;                                 // kierunek: nadawanie
     WIRE1_clr;                                            // wylij 0
     delay_us(14);                                   // czekaj 14 usek
     WIRE1_in;                                     // kierunek: odbir
     delay_us(5);                                     // czekaj 5 usek
     data >>= 1;
     data  |=  ( WIRE1_get<<7  );       // odbir i gromadzenie danych
     delay_us(100);                                 // czekaj 100 usek
   }
   return (data);                                    // odczytane dane
}

// zapis bajtu poprzez interfejs 1-wire ------------------------------
char WIRE1_write_byte(char data) {
   char i;

   for(i=1;i<=8;++i) {                             // wysanie 8-bitw
     WIRE1_out;                                 // kierunek: nadawanie
     WIRE1_clr;                                            // wylij 0
     delay_us(10);                                   // czekaj 10 usek
     if(data & 1 ) {
       WIRE1_set;                                          // wylij 0
     } else {
       WIRE1_clr;                                          // wylij 1
     }
     data >>= 1;
     delay_us(50);                                   // czekaj 50 usek
     WIRE1_set;                    // wylij 1 (zwolnienie magistrali)
     delay_us(50);                                   // czekaj 50 usek
   }
   return(1);
}

// impuls start oraz czekanie na odpowied z 1-wire ------------------
char WIRE1_present() {
    int present;
    WIRE1_out;                                  // kierunek: nadawanie
    WIRE1_clr;                                             // wylij 0
    delay_us(500);                                  // czekaj 500 usek
    WIRE1_in;                                      // kierunek: odbir
    delay_us(65);                                    // czekaj 65 usek
    present= (~WIRE1_get) & 1;                     // odpowied 1-wire
    delay_us(240); 
    return (present);                   // 1  oznacza obecno 1-wire
}

// opnienie czasowe w mikrosekundach -------------------------------
void  delay_us(unsigned n) {
   long i;
   for(i=0;i< (long)(5*n);i++){};   // 3...8
}