/*********************************************************************
MMC.c                        (przykadowe procedury obsugi karty MMC)

Przykadowe procedury obsugi karty multimedialnej MMC. Procedury 
pokazuj jak zainicjowa kart MMC w trybie SPI, jak przesa i 
odebra blok danych o pojemnoci 512 bajtw oraz przeprowadzi odczyt
wielu blokw danych (multiblick). Procedury mog by podstaw do 
napisania innych funkcji obsugi kart MMC.
*********************************************************************/
#include <LPC214x.H>         
#include "INTTYPES.H"            // tu zdefiniowano BYTE, WORD i DWORD
#include "spi.h"
#include "MMC.h"

void static MMC_command( BYTE command, DWORD data);         // komenda
int  static MMC_response(BYTE response);                  // odpowied
int static MMC_wait_for_WR_end( void );        // koniec zapisu do MMC

// --------------------------------------------------- inicjowanie MMC                 

int MMC_Init(void) {       // zakoczenie podprogramu zwraca warto 0
     BYTE i;
          
     IODIR0 |= SPI_SEL;                    // ustaw SSEL1 jako wyjcie
     SSEL1_SET;                                             // SSEL1=1
     //  karta MMC przechodzi w tryb SPI  po wysaniu  10  razy   0xFF
     for (i=0;i<10;i++) SPI_Byte( 0xFF );              // 10 razy 0xFF
     SSEL1_CLR;                                             // SSEL1=0

     MMC_command(0,0);         // CMD0 wprowadza kart MMC w stan idle
     if( MMC_response(0x01) )  {   // czekanie na odpowied 0x01 z MMC
          SSEL1_SET; 
          return 1;                   // brak odpowiedzi
     }
          
     SSEL1_SET;                                             // SSEL1=1
     SPI_Byte(0xFF);       // po zakoczeniu komendy potrzeba poczeka                  

     SSEL1_CLR;                                             // SSEL1=0
     i = 0xFF;                      // komend CDM1 wysya si w ptli
     do {              
         MMC_command( 1,0);                            // komenda CMD1
     }   while ( MMC_response(0x00) && i-- ); // czekanie na odpowied
     
     if ( i == 0 ) {   // jeli upyn czas i nie uzyskano odpowiedzi
         SSEL1_SET; 
         return 2;                          // brak odpowiedzi na CMD1
     }

     SSEL1_SET; 
     SPI_Byte(0xFF);       // po zakoczeniu komendy potrzeba poczeka
     return 0;        // zerowa warto oznacza prawidowe zakoczenie
}

// --------------- ustawienie dugoci bloku do zapisu i odczytu z MMC
int MMC_set_block_length(DWORD block_length) {

     SSEL1_CLR;                                             // SSEL1=0
     MMC_command( 16,block_length);                   // komenda CMD16
     if( MMC_response(0x00) ) {            // ...ustawia dugo bloku
          SSEL1_SET; 
          return 10;                     // nie uzyskano potwierdzenia
     }
     SSEL1_SET; 
     SPI_Byte(0xFF);       // po zakoczeniu komendy potrzeba poczeka
     return 0;        // zerowa warto oznacza prawidowe zakoczenie
}

// ------------------------ odczyt bloku danych (512 bajtw) do bufora
int MMC_RD_block(DWORD block_number, BYTE *MMCRDData) {
     WORD Checksum;

     SSEL1_CLR;                                             // SSEL1=0
     MMC_command( 17, block_number <<9);              // komenda CMD17

     if(MMC_response(0x00)) {     // czekanie na potwierdzenie komendy
          SSEL1_SET; 
          return 20;                     // brak potwierdzenia komendy
     }

     if(MMC_response(0xFE)) {   // blok danych rozpoczyna 'token" 0xFE
          SSEL1_SET;
          return 21;                                // brak bajtu 0xFE
     } 

     SPI_Data(MMCRDData,0,  512  );  // odczyt bloku danych 512 bajtw

     Checksum = SPI_Byte(0xFF);          // blok danych zakoczony CRC
     Checksum = Checksum << 0x08 |  SPI_Byte(0xFF);       // CRC (LSB)
     SSEL1_SET; 
     SPI_Byte(0xFF);       // po zakoczeniu komendy potrzeba poczeka
     return 0;         // zerwa warto oznacza prawidowe zakoczenie
}

// ----------------------------- odczyt wielu blokw: MultiBlock START
// ...    blok danych 1-bajt -   odczyt wielu blokw danych
int MMC_RD_multBl_start(DWORD data_address) {
     SSEL1_CLR;                                             // SSEL1=0
     
     MMC_command( 18,data_address);                   // komenda CMD18
     if(MMC_response(0x00)) {     // czekanie na potwierdzenie komendy
          SSEL1_SET;
          return 30;                             // brak potwierdzenia
     }
     return 0;        // zerowa warto oznacza prawidowe zakoczenie
}

// -------------------------- odczyt wielu blokw: MultiBlock CONTINUE
// ...             blok danych 1-bajt - odczyt kolejnych blokw danych
int MMC_RD_multBl_cont(BYTE *result) {
     WORD Checksum;
     if(MMC_response(0xFE)) {  // czekanie na pocz. bloku "token" 0xFE
          SSEL1_SET;
          return 40;                            // brak pocztku bloku
     } 

     *result  = SPI_Byte(0xFF);      // odbir bloku o dugoci 1 bajt
     Checksum = SPI_Byte(0xFF);                    // odczyt CRC (MSB)
     Checksum = Checksum << 0x08 | SPI_Byte(0xFF); // odczyt CRC (LSB)
     return 0;         // zerwa warto oznacza prawidowe zakoczenie
}

// ---------------------------- odczyt wielu blokw: MultiBlock FINISH
// ...                          koniec odczytu kolejnych blokw danych
int MMC_RD_multBl_fin(void) {
     SSEL1_CLR;                                             // SSEL1=0
     MMC_command( 12,0);             // komenda CMD12: muli block stop

     if( MMC_response(0xFF)  )  {         // czekanie na potwierdzenie
          SSEL1_SET;
          return 50;                             // brak potwierdzenia
     }

     SSEL1_SET;                                        // set SPI SSEL
     SPI_Byte(0xFF);
     return 0;
}
// ---------------------------- zapis bloku danych (512 bajtw) do MMC
int MMC_WR_block(DWORD block_number, BYTE *MMCWRData) {

     SSEL1_CLR;                                             // SSEL1=0
     MMC_command( 24, block_number <<9);              // komenda CMD24

     if(MMC_response(0x00)) {     // czekanie na potwierdzenie komendy
          SSEL1_SET; 
          return 60;                             // brak potwierdzenia
     }

     SPI_Byte(0xFE);            // blok danych rozpoczyna "token" 0xFE
     SPI_Data(0, MMCWRData, 512 );        // wysanie bloku 512 bajtw
     SPI_Byte(0xFF);           // suma kontrolna, bez wyliczenia (MSB)
     SPI_Byte(0xFF);           // suma kontrolna, bez wyliczenia (MSB)

     if ( (SPI_Byte(0xFF) & 0x0F) != 0x05 ) {    // potwierdzenia 0xX5
          SSEL1_SET; 
          return 61;                             // brak potwierdzenia
     }

     if(MMC_wait_for_WR_end()==1) {    // sprawdzenie zajtoci "busy"
          SSEL1_SET; 
          return  62;                                   // bd zapisu
     }
     SSEL1_SET; /* set SPI SSEL */
     SPI_Byte(0xFF);         // po zakoczeniu komendy trzeba poczeka
     return 0;         // zerwa warto oznacza prawidowe zakoczenie
}

// --------------------------------- czekanie na odpowied z karty MMC
int static MMC_response( BYTE response) {
     WORD count = 0xFFF;                     // timeout dla odpowiedzi
     while( ( SPI_Byte(0xFF) != response) && count ) count--;
     return (count == 0 ? 1 : 0 );        // 1 oznacza brak odpowiedzi
}

// -------------------- sprawdzanie zajtoci karty MMC podczas zapisu
int static MMC_wait_for_WR_end( void ){
   WORD count = 0xFFFF;             // timeout dla zapisu do karty MMC
   while( ( SPI_Byte(0xFF) == 0) && count )count--; // 0-czekaj (busy)
   return (count == 0 ? 1 : 0 );          // 1 oznacza brak odpowiedzi
}

// ----------------------- wysanie komendy CMD do katy MMC (6 bajtw)
void static MMC_command( BYTE command, DWORD data) {
  BYTE i;
  SPI_Byte( 0x40 | command );         // komenda zaczyna si bitami 01
  for(i=4;i;i--)   SPI_Byte( GET_BYTE(data,i-1) );     // bajty danych
  SPI_Byte( 0x95 );               // tylko CMD0 ma sum kontroln 0x95
}