/*********************************************************************
IDLE_s.c                                         (prosty program IDLE)

Program demonstruje uycie energooszczdnego trybu IDLE 
mikrokontrolera. Do wyprowadzenia procesora ze stanu IDLE mona 
uy dowolnego przerwania, w tym przypadku przerwania EINT1 (P0.14).
Procedura przerwania jest pusta bo suy tyko do powrotu ze 
stanu IDLE. W stanie IDLE wieci co druga dioda LED piru P1.
*********************************************************************/
#include <lpc214x.h>

#define NBit_LED  16                      // przesunicie dla diod LED
#define M        0xFF                       //maska dla 8-miu diod LED

#define LEDs_init IO1DIR = M << NBit_LED

void int_handler (void)  __irq;

__inline void LEDs_put(char x) { //-----------------------------------
  IO1CLR = ( ~x & M ) << NBit_LED ; 
  IO1SET = (  x & M ) << NBit_LED ; 
}

__inline void delay(void) {      //-----------------------------------
   unsigned j;
   for(j = 0; j < 0x200000; j++) ;
}

//--------------------------------------------------------------------
main(void) {
    int i;
    LEDs_init;                                   // inicjalizacja diod

    EXTMODE = 0;         //   EXTMODE = 2 przerwanie wyzwalane zboczem
    EXTPOLAR = 0;                   // EXTPOLAR = 2 zbocze narastajce
    PINSEL0 = 2 << 28;                       // wybranie pinu do EINT1

    VICVectAddr1 = (unsigned long) int_handler;  // obsuga przerwania
    VICVectCntl1 = 0x20 | 15; // wybr szczeliny slot i wektora przerw.
    VICIntEnable = 1 << 15;                      // wczenie przerwa

    PCONP= 0;          // wyczenie wszystkich urzdze peryferyjnych

    while(1) {
        for(i=1;i<0x200;i<<=1) {
             LEDs_put(i);      // wywietlenie zawartoci i na diodach
            delay() ;                            // opnienie czasowe
        }

        LEDs_put(0x55);                   // co druga dioda LED wieci
        PCON = 1;                     // przejcie do trybu power IDLE
        delay() ;                                // opnienie czasowe
    }
}

//--------------------------------------------------------------------
void int_handler() __irq {// procedura obsugi przerwania zewntrznego
    EXTINT = 2;                   // kasowanie przerwania zewntrznego
    VICVectAddr = 0;            // skasowanie przerwania w kontrolerze
}
