/************************************************************************
LCD8bit.c                      (biblioteka LCD dla 8-bitowego interfejsu)

Biblioteka obsugi wywietlacza LCD podczonego do portu poprzez 8-linii danych.
Wywietlacz LCD podczony jest do portu P1:
   LCD          Procesor
   -------------------
   DB0        = P1.24
   DB1        = P1.25
   DB2        = P1.26
   DB3        = P1.27
   DB4        = P1.28
   DB5        = P1.29
   DB6        = P1.30
   DB7        = P1.31

   D/C~ (RS)  = P1.21        Data/Control~
   R/W~       = P1.22        Read/Write~
   E          = P1.23        Enable
************************************************************************/
#include <LPC214x.H>              // zbir definiujcy rejestry procesora
#include "LCD8bit.h"                                     // definicje LCD
#define INTERFACE LCD_8BIT 

//------------------------------------ zapis 8-bitw do wywietlacza LCD 
void write_char ( char val) {
  E_LCD_clr;                                       // Enable=0 nieaktywne
  RW_LCD_clr;                                      // linia zapisu R/W~=0
  E_LCD_set;                   // linia Enable E=1: zapis do wywietlacza
  DATA_to_LCD(val);                       // wystawienie danych do zapisu
  E_LCD_clr;                           // linia Enable E=0: koniec zapisu
}

//------------------- zapis sowa sterujcego Control do wywietlacza LCD
void LCD_ctrlWR ( char cmde) {
    unsigned var;
    WAIT;                                                     // poczekaj
    DC_LCD_clr;                            // wybierz rejestr Control LCD
    write_char(cmde);                // zapis bajtu polecenia sterujcego
}

//--------------------------------- zapis danych Data do wywietlacza LCD
void LCD_dataWR ( char byte) {
    unsigned var;
    WAIT;                                                     // poczekaj
    DC_LCD_set;                        // wybierz rejestr danych Data LCD
    write_char(byte);                               // zapis bajtu danych
}

 // ------------------------ sprawdzenie flagi zajtoci wywietlacza LCD
char read_char (void) {
    char val;
    RW_LCD_set;
    E_LCD_clr; 
    E_LCD_set; 
    val = DATA_from_LCD() ;   // najstarszy bit informuje o zajtoci LCD
    E_LCD_clr; 
    return (val);
}

//-------------------------------- odczyt danych Data do wywietlacza LCD
char LCD_dataRD ( void) {
    unsigned var;
    WAIT;                                                  // poczekaj
    DC_LCD_set;                     // wybierz rejestr danych Data LCD
    var= read_char();                           // zapis  bajtu danych
    return (var);
}

//---------------------------- odczyt sowa Control z wywietlacza LCD
char LCD_ctrlRD ( void) {
    unsigned var;
    WAIT;                                                  // poczekaj
    DC_LCD_clr;                     // wybierz rejestr danych Data LCD
    var= read_char();                           // zapis  bajtu danych
    return (var);
}

// ---------------------- sprawdzenie flagi zajtoci wywietlacza LCD
char check_busy (void) {
    char bf;
    DC_LCD_clr;        // linia D/C~=0 : odczyt sowa Control (status)
    bf = read_char() & 0x80;   // najstarszy bit informuje o zajtoci
    return (bf);
}  

#include "lcd_comm.c"               // <<<<<<<<<<<<<<< wcz (include)
