/*********************************************************************
AT25040.c                      (biblioteka moduw pamici SPI EEPROM)
Biblioteka moduw obsugi pamici SPI typu AT25010, AT25020 i
At25040. W zalenoci od producenta pamici moliwe jest programowanie
pojedynczych bajtw i blokw pamici.
*********************************************************************/
#include <LPC214x.H> 
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD
#include "spi.h"
#include "AT25040.h"

 
// -------------------------------------------- odczyt statusu AT25040
BYTE AT25040_read_status(void )  {
     unsigned char spi_data;
     AT25040_CS_clr();                   // chip select - wybr ukadu
     SPI_Byte(AT25040_CMD_RDSR);
     spi_data=SPI_Byte(0x55);                       // dowolna warto
     AT25040_CS_set(); 
     return (spi_data);
}

// -----------------------------  prawdzenie gotowoci pamici AT25040
BYTE AT25040_ready(void)  {
     return ( AT25040_read_status() & AT25040_MSK_RDY) ;
} 

// -------------------------- ustawienie gody na zapis pamici AT25040
void AT25040_set_write_enable(void )  {
     AT25040_WAIT;
     AT25040_CS_clr();                      
     SPI_Byte(AT25040_CMD_WREN);
     AT25040_CS_set(); 
}


// -------------------------- ustawienie gody na zapis pamici AT25040
void AT25040_set_status(void )  {
     AT25040_WAIT;
     AT25040_CS_clr();                      
     SPI_Byte(AT25040_CMD_WRSR);
     SPI_Byte(0x02);
     AT25040_CS_set(); 
}

// -------------------------- kasowanie zgody na zapis pamici AT25040
void AT25040_clear_write_enable(void ) {
     AT25040_WAIT;
     AT25040_CS_clr();
     SPI_Byte(AT25040_CMD_WRDI);
     AT25040_CS_set(); 
}

// ------------------------------------ zapis bajtu do pamici AT25040
void AT25040_WR_byte(BYTE adr, BYTE to_write )  {
     AT25040_WAIT;
    AT25040_WR_ENABLE;
     AT25040_CS_clr();
    SPI_Byte(AT25040_CMD_WRITE);
    SPI_Byte(adr);
    SPI_Byte(to_write);
     AT25040_CS_set(); 
}

// ------------------------------------ odczyt bajtu z pamici AT25040
BYTE AT25040_RD_byte(BYTE adr )  {
     BYTE spi_data;
    AT25040_WAIT;
     AT25040_CS_clr();
    SPI_Byte(AT25040_CMD_READ);
     SPI_Byte(adr);
     spi_data=SPI_Byte(0xff);                       // dowolna warto
     AT25040_CS_set(); 
     return (spi_data);
}

// ----------------------------------- zapis danych do pamici AT25040
void AT25040_WR_data( BYTE adr, BYTE  *ptr_data, BYTE nb_data) {
    adr     &= ~(AT25040_MAX_PAGE -1);       //zapis na strony pamici
     nb_data &=   AT25040_MAX_PAGE -1 ;
    AT25040_WAIT;
    AT25040_WR_ENABLE;
     AT25040_CS_clr();
     SPI_Byte(AT25040_CMD_WRITE);
     SPI_Byte(adr); 
	 SPI_Data( 0,ptr_data, nb_data );    
     AT25040_CS_set(); 
}

// ------------------------------odczyt cigu danych z pamici AT25040
void AT25040_RD_data( BYTE adr, BYTE  *ptr_data, BYTE nb_data) {
     AT25040_WAIT;
    AT25040_CS_clr();
     SPI_Byte(AT25040_CMD_READ);
     SPI_Byte(adr);
	 SPI_Data( ptr_data,0, nb_data );   
     AT25040_CS_set(); 
}
