/*********************************************************************
SPI1_soft.c                                   (programowa obsuga SPI)
                             
Biblioteka moduw obsugi SPI metod programow w procesorach '21xx.
Kady z podprogramw jednoczenie zapisuje i odczytuje dane. Modu
SPI_Byte - nadaje i odbiera bajt danych
SPI_Data - nadaje i odbiera cig bajtw.
*********************************************************************/
#include <LPC214x.H> 
#include "inttypes.h"            // tu zdefiniowano BYTE, WORD i DWORD

#define SCK_NBit    4   // p0.4 SCK0           p0.17 SCK1
#define MOSI_NBit   6   // p0.6 MOSI0          p0.19 MOSI1
#define MISO_NBit   5   // p0.5 MISO0          p0.18 MISO1                                                                                                                   MISO1

#define spi_sck_set()    IOSET0 = 1  << SCK_NBit  
#define spi_sck_clear()  IOCLR0 = 1  << SCK_NBit  
#define spi_mosi_set()   IOSET0 = 1  << MOSI_NBit 
#define spi_mosi_clear() IOCLR0 = 1  << MOSI_NBit  
#define spi_miso_get()   IOPIN0 & (1 << MISO_NBit)

//-------------------------------------------------  Przygotowanie SPI
void SPI_Init (void) {
   IODIR0 |=  (1  << SCK_NBit) | (1 << MOSI_NBit);  
}

// -------------------- Podprogram nadawania i odbioru bajtu przez SPI
BYTE SPI_Byte(BYTE to_transmit)  {
   BYTE i,receive=0;
   spi_sck_clear();                     // sygna SCK zaczyna si od 0

   for(i=8;i;i--)       {             // 8 bitw do wysania i odbioru
                           
   if (( to_transmit & 0x80)) spi_mosi_set();     // zapis bitu do SPI
        else spi_mosi_clear();
     to_transmit <<=1;                          // przesunicie w lewo
    spi_sck_set();                           // zbocze narastajce SCK
                              
    receive <<=1;                               // przesunicie w lewo
    if (spi_miso_get()) receive|=1;               // odczyt bitu z SPI
       spi_sck_clear();                        // zbocze opadajce SCK
  };
  return receive;
}
// ------------- Podprogram nadawania i odbioru cigu bajtw przez SPI
void SPI_Data( BYTE *receive,BYTE *send, WORD Length ) {
  DWORD i;
  BYTE  *tmp = receive;
  for ( i = 0; i < Length; i++ ) {
    if(send==0)     *(receive++) = SPI_Byte(0xFF);     // tylko odczyt
    if(receive==0)  SPI_Byte(*(send++));            // tylko nadawanie
    if ( (send !=0) &&  (receive!=0)) 
                                   *(receive++) = SPI_Byte(*(send++));
  }
  receive=tmp;      // ustaw wskanik na pocztek odebranej informacji
  return;
}
