/***************************************************************************
noise_F.c             (generacja szumu przez generator liczb pseudolosowych)

Program generuje szum pseudolosowy dwoma metodami. Pierwsza metoda 
to symulacja rejestru o sprzniu liniowym LSFR typu "wiele-na-jeden"
(typu Fibonacci). Druga to uycie funkcji rand() ze standardowej biblioteki.
Szum syszalny jest powstaje na wyjciu przetwornika DAC i mona go
usysze w goniku a take moe by obserwowany na diodzie LED 
(po obnieniu czstotliwoci). Czstotliwo wysyania prbek na wyjcie to
ok. 70kHz. Przygotowano dwa wielomiany generujce: 16 i 32 bitowy. 
Wielomiany zapisywane s w postaci naturalnej, nie odwrconej. Osiem 
pierwszych prbek sygnau jest drukowana. 
***************************************************************************/
#include <LPC214x.H>      
#include <stdlib.h>
#include <stdio.h>
#include "inttypes.h"                  // tu zdefiniowano BYTE, WORD i DWORD
#include "UART.h"

#define   Poly_Degree  16                  // 16, 32  stopie wielomianu CRC

//------------------------------------------------------ wielomian 16-bitowy
#define     Polynom   0xA011         // LSFR-16:   x^16 +x^15 +x^13 + x^4 +1

//------------------------------------------------------ wielomian 32-bitowy
//#define   Polynom   0x00400007;    // LSFR-32:    x^32 +x^22 +x^2 + x^1 +1    

//----------------------------------------------------- sterowanie diod LED
#define NBit_LED 23                          // dioda LED sterowana bitem 16
#define LED_init IO1DIR |=  (1 << NBit_LED)               // bit jako OUTPUT
#define LED_clr  IO1CLR = 1 << NBit_LED                 // wyzeruj diod LED
#define LED_set  IO1SET = 1 << NBit_LED                   // wcz diod LED

//--------------------------------------------------------------------------
main(void) {
 DWORD  x,parity;
 DWORD i,j=0;
 WORD CRC=0xFFFF;       // pocztkowy stan CRC (preset)dla liczb 16-bitowych
 //DWORD CRC=0xFFFFffff;     //pocztkowy stan (preset)dla liczb 32-bitowych


 PINSEL1 |= 1<<19;                        // wyjcie przetwornika DAC (AOUT)
 LED_init;                                
 UART_init; 
 printf("==========================\n");
 while(1) {
    if (CRC & 0x1) LED_set; else LED_clr;     // kady bit CRC generuje szum
    // DACR= ( rand() & 0xFFc0 );                     // funkcja standardowa
    DACR= CRC & 0xFFc0;                          // przesanie prbki do DAC
    x= CRC & Polynom ;              // wybrane bity bior udzia w generacji

     parity=0;      
     for (i=0;i<Poly_Degree;i++) {
            if( x & 1 ) parity ^= 1;  // obliczenie rnicy symetrycznej XOR
            x >>=1;
     }
                                        
     if(   parity  )                              // symulacja rejestru LSFR
            CRC = (CRC>>1) | (1<< (Poly_Degree-1)); else CRC = CRC>>1;
     if (CRC & 0x1) LED_set ; else LED_clr;                     // dioda LED
     //for (i=0;i<0x10000;i++);                  // opnienie dla diody LED
     //for (i=0;i<0x100;i++);       // opnienie dla wielomianu 16-bitowego
     if (j++<8 )printf(" %8x \n", CRC);   // drukowanie 8-miu prbek sygnau     
 }
}
