/*********************************************************************
MMC_RW_512.c                      (zapis i odczyt blokw po 512bajtw)

Zapis i odczyt wielu blokw danych o objtoci 512bajtw. Zapisywane
dane mog by odtworzone w postaci przebiegw: rampa,przebieg 
prostoktny,szum, sinus, sygna o zmiennej czstotliwoci chrp.
*********************************************************************/
#include <LPC213X.H> // procesory '213x i '214x tak samo obsuguj SPI
#include <stdio.H>
#include <math.H>
#include "inttypes.h"
#include "spi.h"
#include "MMC.h"
#include "uart.h"

#define BLOCK_SIZE   512                       // rozmiar bloku danych

#define BLOCK_START  0x1000 // 0xF500 max. for 32MB,(0x1EA00 for 64MB)
#define BLOCK_END    0x1200     
void message (const char *message, BYTE Status);

BYTE MMCWRData[BLOCK_SIZE];                  // bufor danych do zapisu
BYTE MMCRDData[BLOCK_SIZE];                // bufor danych odczytanych
BYTE Status;

//--------------------------------------------------------------------
void main (void) {
  DWORD i, BlockIndex = 0;

  UART_init;                      // inicjowanie transmisji szeregowej
  PINSEL1 |= 1<<19;                              // wybierz DAC (AOUT)
  SPI_Init();                                       // inicjowanie SPI
  printf("MMC Audio WRITE/READ  512 bytes  ----------------------\n");
  if ( MMC_Init() )                  message("INIT/Config Error\n",0);
  if ( MMC_set_block_length(BLOCK_SIZE ) )
                                message("Set Block Length Error\n",0);

  for( BlockIndex = BLOCK_START; BlockIndex< BLOCK_END; BlockIndex++){
   // for ( i = 0; i<BLOCK_SIZE ; i++ ) MMCWRData[i]= i;      // rampa
   // for ( i = 0; i<BLOCK_SIZE ; i++ ) MMCWRData[i]= i&1? 0xff: 0x00;
   // for ( i = 0; i<BLOCK_SIZE ; i++ ) MMCWRData[i]= (BYTE)rand(); 
   // for ( i = 0; i<BLOCK_SIZE ; i++ ) MMCWRData[i]=
                        0x7f*sin(2*3.141596*(i&0xF)/16)+0x80;  //sinus
   for    ( i = 0; i<BLOCK_SIZE ; i++ ) MMCWRData[i]=
                        0x7f*sin(2*3.141596*i*i/2024/4)+0x80; // chirp

   if ( MMC_WR_block(BlockIndex ,MMCWRData))     // zapis bloku danych
                              message("Block write:  Error\n",Status);
   if ( MMC_RD_block (BlockIndex ,MMCRDData))   // odczyt bloku danych
                              message("Block Read:   Error\n",Status);
   printf ("Block # %x\n",BlockIndex);
   for ( i = 0; i<BLOCK_SIZE  ; i++ ){
         printf("%02x ", MMCRDData[i]  );               // drukuj dane
         if ( (i %16) ==15) printf("\n");   // nowa linia co 16-bajtw
   }
  }
}

// ----------------------------------- drukowanie komunikatu o bdzie
void message (const char *message, BYTE Status) {
    if(Status !=0) printf(" Status %d\n", Status);
    printf(message);
    while ( 1 );                                                //stop        
}
